from __future__ import annotations

import collections.abc
import typing

import amulet.nbt

__all__: list[str] = ["DataVersionComponent", "JavaRawChunkComponent"]

class DataVersionComponent:
    ComponentID: typing.ClassVar[str] = "Amulet::DataVersionComponent"
    @property
    def data_version(self) -> int: ...

class JavaRawChunkComponent:
    ComponentID: typing.ClassVar[str] = "Amulet::JavaRawChunkComponent"
    @property
    def raw_data(self) -> collections.abc.MutableMapping[str, amulet.nbt.NamedTag]:
        """
        This is subject to change as data gets moved into the chunk class.
        Do not rely on data in here existing.
        """

    @raw_data.setter
    def raw_data(
        self, arg1: collections.abc.Mapping[str, amulet.nbt.NamedTag]
    ) -> None: ...



from __future__ import annotations

import typing

import amulet.level.bedrock.raw_chunk

__all__: list[str] = ["BedrockRawChunkComponent"]

class BedrockRawChunkComponent:
    ComponentID: typing.ClassVar[str] = "Amulet::BedrockRawChunkComponent"
    @property
    def raw_data(self) -> amulet.level.bedrock.raw_chunk.BedrockRawChunk:
        """
        This is subject to change as data gets moved into the chunk class.
        Do not rely on data in here existing.
        """

    @raw_data.setter
    def raw_data(
        self, arg1: amulet.level.bedrock.raw_chunk.BedrockRawChunk
    ) -> None: ...



from __future__ import annotations

import amulet.core.chunk
import amulet.core.chunk.component.block_component
import amulet.level.bedrock.chunk_components

__all__: list[str] = ["BedrockChunk", "BedrockChunk118"]

class BedrockChunk(amulet.core.chunk.Chunk):
    pass

class BedrockChunk118(
    BedrockChunk,
    amulet.level.bedrock.chunk_components.BedrockRawChunkComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    pass



from __future__ import annotations

import typing

import amulet.core.biome
import amulet.core.block
import amulet.core.selection.box
import amulet.level.bedrock.chunk
import amulet.level.bedrock.raw_chunk
import amulet.utils.lock

__all__: list[str] = ["BedrockChunkCoordIterator", "BedrockRawDimension"]

class BedrockChunkCoordIterator:
    def __iter__(self) -> typing.Any: ...
    def __next__(self) -> tuple[int, int]: ...

class BedrockRawDimension:
    def decode_chunk(
        self,
        raw_chunk: amulet.level.bedrock.raw_chunk.BedrockRawChunk,
        cx: typing.SupportsInt,
        cz: typing.SupportsInt,
    ) -> amulet.level.bedrock.chunk.BedrockChunk:
        """
        Decode a raw chunk to a chunk object.
        TODO: thread safety
        """

    def delete_chunk(self, cx: typing.SupportsInt, cz: typing.SupportsInt) -> None:
        """
        Delete the chunk from this dimension.
        External ReadWrite:SharedReadWrite lock required.
        """

    def destroy(self) -> None:
        """
        Destroy the instance.
        Calls made after this will fail.
        This may only be called by the owner of the instance.
        External ReadWrite:Unique lock required.
        """

    def encode_chunk(
        self,
        chunk: amulet.level.bedrock.chunk.BedrockChunk,
        cx: typing.SupportsInt,
        cz: typing.SupportsInt,
    ) -> amulet.level.bedrock.raw_chunk.BedrockRawChunk:
        """
        Encode a chunk object to its raw data.
        This will mutate the chunk data.
        TODO: thread safety
        """

    def get_chunk(
        self, cx: typing.SupportsInt, cz: typing.SupportsInt
    ) -> amulet.level.bedrock.chunk.BedrockChunk:
        """
        Get and decode the chunk.
        TODO: thread safety
        """

    def get_raw_chunk(
        self, cx: typing.SupportsInt, cz: typing.SupportsInt
    ) -> amulet.level.bedrock.raw_chunk.BedrockRawChunk:
        """
        Get the raw chunk from this dimension.
        External Read:SharedReadWrite lock required.
        """

    def has_chunk(self, cx: typing.SupportsInt, cz: typing.SupportsInt) -> bool:
        """
        Does the chunk exist in this dimension.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    def is_destroyed(self) -> bool:
        """
        Has the instance been destroyed.
        If this is false, other calls will fail.
        External Read:SharedReadWrite lock required.
        """

    def set_chunk(
        self,
        cx: typing.SupportsInt,
        cz: typing.SupportsInt,
        chunk: amulet.level.bedrock.chunk.BedrockChunk,
    ) -> None:
        """
        Encode and set the chunk.
        This will mutate the chunk data.
        TODO: thread safety
        """

    def set_raw_chunk(
        self,
        cx: typing.SupportsInt,
        cz: typing.SupportsInt,
        chunk: amulet.level.bedrock.raw_chunk.BedrockRawChunk,
    ) -> None:
        """
        Set the chunk in this dimension from raw data.
        External ReadWrite:SharedReadWrite lock required.
        """

    @property
    def all_chunk_coords(self) -> BedrockChunkCoordIterator:
        """
        An iterator of all chunk coordinates in the dimension.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    @property
    def bounds(self) -> amulet.core.selection.box.SelectionBox:
        """
        The selection box that fills the whole world.
        Thread safe.
        """

    @property
    def default_biome(self) -> amulet.core.biome.Biome:
        """
        The default biome for this dimension.
        Thread safe.
        """

    @property
    def default_block(self) -> amulet.core.block.BlockStack:
        """
        The default block for this dimension.
        Thread safe.
        """

    @property
    def dimension_id(self) -> str:
        """
        The identifier for this dimension. eg. "minecraft:overworld".
        Thread safe.
        """

    @property
    def internal_dimension_id(self) -> int:
        """
        The internal identifier for this dimension. eg 0, 1 or 2
        Thread safe.
        """

    @property
    def lock(self) -> amulet.utils.lock.OrderedLock:
        """
        The public lock
        Thread safe.
        """



from __future__ import annotations

import typing
from builtins import str as DimensionId

import amulet.core.biome
import amulet.core.block
import amulet.core.selection.box
import amulet.core.selection.box_group
import amulet.level.abc.chunk_handle

__all__: list[str] = ["Dimension", "DimensionId"]

class Dimension:
    def get_chunk_handle(
        self, cx: typing.SupportsInt, cz: typing.SupportsInt
    ) -> amulet.level.abc.chunk_handle.ChunkHandle:
        """
        Get the chunk handle for the given chunk in this dimension.
        Thread safe.

        :param cx: The chunk x coordinate to load.
        :param cz: The chunk z coordinate to load.
        """

    @property
    def bounds(
        self,
    ) -> (
        amulet.core.selection.box.SelectionBox
        | amulet.core.selection.box_group.SelectionBoxGroup
    ):
        """
        The editable region of the dimension.
        Thread safe.
        """

    @property
    def default_biome(self) -> amulet.core.biome.Biome:
        """
        The default biome for this dimension
        Thread safe.
        """

    @property
    def default_block(self) -> amulet.core.block.BlockStack:
        """
        The default block for this dimension.
        Thread safe.
        """

    @property
    def dimension_id(self) -> str:
        """
        Get the dimension id for this dimension.
        Thread safe.
        """



from __future__ import annotations

from builtins import str as BedrockInternalDimensionID

from amulet.level.bedrock.chunk_handle import BedrockChunkHandle
from amulet.level.bedrock.dimension import BedrockDimension
from amulet.level.bedrock.level import BedrockLevel
from amulet.level.bedrock.level_dat import BedrockLevelDat
from amulet.level.bedrock.raw_dimension import BedrockRawDimension
from amulet.level.bedrock.raw_level import BedrockRawLevel

from . import (
    chunk,
    chunk_components,
    chunk_handle,
    dimension,
    level,
    level_dat,
    raw_chunk,
    raw_dimension,
    raw_level,
)

__all__: list[str] = [
    "BedrockChunkHandle",
    "BedrockDimension",
    "BedrockInternalDimensionID",
    "BedrockLevel",
    "BedrockLevelDat",
    "BedrockRawDimension",
    "BedrockRawLevel",
    "chunk",
    "chunk_components",
    "chunk_handle",
    "dimension",
    "level",
    "level_dat",
    "raw_chunk",
    "raw_dimension",
    "raw_level",
]



from __future__ import annotations

import typing

import amulet.core.biome
import amulet.core.block
import amulet.core.chunk
import amulet.core.chunk.component.block_component
import amulet.level.java.chunk_components

__all__: list[str] = [
    "JavaChunk",
    "JavaChunk0",
    "JavaChunk1444",
    "JavaChunk1466",
    "JavaChunk2203",
    "JavaChunkNA",
]

class JavaChunk(amulet.core.chunk.Chunk):
    pass

class JavaChunk0(
    JavaChunk,
    amulet.level.java.chunk_components.JavaRawChunkComponent,
    amulet.level.java.chunk_components.DataVersionComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    def __init__(
        self,
        data_version: typing.SupportsInt,
        default_block: amulet.core.block.BlockStack,
        default_biome: amulet.core.biome.Biome,
    ) -> None: ...

class JavaChunk1444(
    JavaChunk,
    amulet.level.java.chunk_components.JavaRawChunkComponent,
    amulet.level.java.chunk_components.DataVersionComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    def __init__(
        self,
        data_version: typing.SupportsInt,
        default_block: amulet.core.block.BlockStack,
        default_biome: amulet.core.biome.Biome,
    ) -> None: ...

class JavaChunk1466(
    JavaChunk,
    amulet.level.java.chunk_components.JavaRawChunkComponent,
    amulet.level.java.chunk_components.DataVersionComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    def __init__(
        self,
        data_version: typing.SupportsInt,
        default_block: amulet.core.block.BlockStack,
        default_biome: amulet.core.biome.Biome,
    ) -> None: ...

class JavaChunk2203(
    JavaChunk,
    amulet.level.java.chunk_components.JavaRawChunkComponent,
    amulet.level.java.chunk_components.DataVersionComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    def __init__(
        self,
        data_version: typing.SupportsInt,
        default_block: amulet.core.block.BlockStack,
        default_biome: amulet.core.biome.Biome,
    ) -> None: ...

class JavaChunkNA(
    JavaChunk,
    amulet.level.java.chunk_components.JavaRawChunkComponent,
    amulet.level.java.chunk_components.DataVersionComponent,
    amulet.core.chunk.component.block_component.BlockComponent,
):
    def __init__(
        self,
        default_block: amulet.core.block.BlockStack,
        default_biome: amulet.core.biome.Biome,
    ) -> None: ...



import os
from packaging.version import Version
import get_compiler

AMULET_COMPILER_TARGET_REQUIREMENT = "==2.0"

PYBIND11_REQUIREMENT = "==3.0.1"
AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = "~=1.2.0.0a2"
AMULET_IO_REQUIREMENT = "~=2.0.0.0a0"
AMULET_LEVELDB_REQUIREMENT = "~=3.0.5.0a0"
AMULET_UTILS_REQUIREMENT = "~=1.1.3.0a8"
AMULET_ZLIB_REQUIREMENT = "~=1.0.8.0a0"
AMULET_NBT_REQUIREMENT = "~=5.0.3.0a0"
AMULET_CORE_REQUIREMENT = "~=2.0.8.0a0"
AMULET_GAME_REQUIREMENT = "~=1.0.4.0a0"
AMULET_ANVIL_REQUIREMENT = "~=1.0.4.0a0"
NUMPY_REQUIREMENT = "~=2.0"
PIL_REQUIREMENT = "~=11.3"

if os.environ.get("AMULET_PYBIND11_EXTENSIONS_REQUIREMENT", None):
    AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = f"{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT},{os.environ['AMULET_PYBIND11_EXTENSIONS_REQUIREMENT']}"

if os.environ.get("AMULET_IO_REQUIREMENT", None):
    AMULET_IO_REQUIREMENT = (
        f"{AMULET_IO_REQUIREMENT},{os.environ['AMULET_IO_REQUIREMENT']}"
    )

if os.environ.get("AMULET_LEVELDB_REQUIREMENT", None):
    AMULET_LEVELDB_REQUIREMENT = (
        f"{AMULET_LEVELDB_REQUIREMENT},{os.environ['AMULET_LEVELDB_REQUIREMENT']}"
    )

if os.environ.get("AMULET_UTILS_REQUIREMENT", None):
    AMULET_UTILS_REQUIREMENT = (
        f"{AMULET_UTILS_REQUIREMENT},{os.environ['AMULET_UTILS_REQUIREMENT']}"
    )

if os.environ.get("AMULET_ZLIB_REQUIREMENT", None):
    AMULET_ZLIB_REQUIREMENT = (
        f"{AMULET_ZLIB_REQUIREMENT},{os.environ['AMULET_ZLIB_REQUIREMENT']}"
    )

if os.environ.get("AMULET_NBT_REQUIREMENT", None):
    AMULET_NBT_REQUIREMENT = (
        f"{AMULET_NBT_REQUIREMENT},{os.environ['AMULET_NBT_REQUIREMENT']}"
    )

if os.environ.get("AMULET_CORE_REQUIREMENT", None):
    AMULET_CORE_REQUIREMENT = (
        f"{AMULET_CORE_REQUIREMENT},{os.environ['AMULET_CORE_REQUIREMENT']}"
    )

if os.environ.get("AMULET_GAME_REQUIREMENT", None):
    AMULET_GAME_REQUIREMENT = (
        f"{AMULET_GAME_REQUIREMENT},{os.environ['AMULET_GAME_REQUIREMENT']}"
    )

if os.environ.get("AMULET_ANVIL_REQUIREMENT", None):
    AMULET_ANVIL_REQUIREMENT = (
        f"{AMULET_ANVIL_REQUIREMENT},{os.environ['AMULET_ANVIL_REQUIREMENT']}"
    )


def get_specifier_set(version_str: str) -> str:
    """
    version_str: The PEP 440 version number of the library.
    """
    version = Version(version_str)
    if version.epoch != 0 or version.is_devrelease or version.is_postrelease:
        raise RuntimeError(f"Unsupported version format. {version_str}")

    return f"~={version.major}.{version.minor}.{version.micro}.0{''.join(map(str, version.pre or ()))}"


AMULET_COMPILER_VERSION_REQUIREMENT = get_compiler.main()


try:
    import amulet.pybind11_extensions
except ImportError:
    pass
else:
    AMULET_PYBIND11_EXTENSIONS_REQUIREMENT = get_specifier_set(
        amulet.pybind11_extensions.__version__
    )

try:
    import amulet.io
except ImportError:
    pass
else:
    AMULET_IO_REQUIREMENT = get_specifier_set(amulet.io.__version__)

try:
    import amulet.leveldb
except ImportError:
    pass
else:
    AMULET_LEVELDB_REQUIREMENT = get_specifier_set(amulet.leveldb.__version__)

try:
    import amulet.utils
except ImportError:
    pass
else:
    AMULET_UTILS_REQUIREMENT = get_specifier_set(amulet.utils.__version__)

try:
    import amulet.zlib
except ImportError:
    pass
else:
    AMULET_ZLIB_REQUIREMENT = get_specifier_set(amulet.zlib.__version__)

try:
    import amulet.nbt
except ImportError:
    pass
else:
    AMULET_NBT_REQUIREMENT = get_specifier_set(amulet.nbt.__version__)

try:
    import amulet.core
except ImportError:
    pass
else:
    AMULET_CORE_REQUIREMENT = get_specifier_set(amulet.core.__version__)

try:
    import amulet.game
except ImportError:
    pass
else:
    AMULET_GAME_REQUIREMENT = get_specifier_set(amulet.game.__version__)

try:
    import amulet.anvil
except ImportError:
    pass
else:
    AMULET_ANVIL_REQUIREMENT = get_specifier_set(amulet.anvil.__version__)


def get_build_dependencies() -> list:
    return [
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
        f"amulet-io{AMULET_IO_REQUIREMENT}",
        f"amulet-leveldb{AMULET_LEVELDB_REQUIREMENT}",
        f"amulet-utils{AMULET_UTILS_REQUIREMENT}",
        f"amulet-zlib{AMULET_ZLIB_REQUIREMENT}",
        f"amulet-nbt{AMULET_NBT_REQUIREMENT}",
        f"amulet-core{AMULET_CORE_REQUIREMENT}",
        f"amulet-game{AMULET_GAME_REQUIREMENT}",
        f"amulet-anvil{AMULET_ANVIL_REQUIREMENT}",
    ] * (not os.environ.get("AMULET_SKIP_COMPILE", None))


def get_runtime_dependencies() -> list[str]:
    return [
        f"amulet-compiler-target{AMULET_COMPILER_TARGET_REQUIREMENT}",
        f"amulet-compiler-version{AMULET_COMPILER_VERSION_REQUIREMENT}",
        f"pybind11{PYBIND11_REQUIREMENT}",
        f"amulet-pybind11-extensions{AMULET_PYBIND11_EXTENSIONS_REQUIREMENT}",
        f"amulet-io{AMULET_IO_REQUIREMENT}",
        f"amulet-leveldb{AMULET_LEVELDB_REQUIREMENT}",
        f"amulet-utils{AMULET_UTILS_REQUIREMENT}",
        f"amulet-zlib{AMULET_ZLIB_REQUIREMENT}",
        f"amulet-nbt{AMULET_NBT_REQUIREMENT}",
        f"amulet-core{AMULET_CORE_REQUIREMENT}",
        f"amulet-game{AMULET_GAME_REQUIREMENT}",
        f"amulet-anvil{AMULET_ANVIL_REQUIREMENT}",
        f"numpy{NUMPY_REQUIREMENT}",
        f"pillow{PIL_REQUIREMENT}",
    ]



from __future__ import annotations

import collections.abc
import typing

import amulet.core.chunk
import amulet.level.abc.chunk_handle
import amulet.level.java.chunk

__all__: list[str] = ["JavaChunkHandle"]

class JavaChunkHandle(amulet.level.abc.chunk_handle.ChunkHandle):
    def get_chunk(
        self, component_ids: collections.abc.Iterable[str] | None = None
    ) -> amulet.level.java.chunk.JavaChunk:
        """
        Get a unique copy of the chunk data.
        """

    @typing.overload
    def set_chunk(self, chunk: amulet.level.java.chunk.JavaChunk) -> None:
        """
        Overwrite the chunk data.
        You must acquire the chunk lock before setting.
        If you want to edit the chunk, use :meth:`edit` instead.

        :param chunk: The chunk data to set.
        """

    @typing.overload
    def set_chunk(self, chunk: amulet.core.chunk.Chunk) -> None:
        """
        Overwrite the chunk data.
        You must acquire the chunk lock before setting.
        If you want to edit the chunk, use :meth:`edit` instead.

        :param chunk: The chunk data to set.
        """



from __future__ import annotations

import os
import typing

import amulet.nbt

__all__: list[str] = ["BedrockLevelDat"]

class BedrockLevelDat:
    named_tag: amulet.nbt.NamedTag
    @staticmethod
    def from_binary(buffer: bytes) -> BedrockLevelDat: ...
    @staticmethod
    def from_file(path: os.PathLike | str | bytes) -> BedrockLevelDat: ...
    def __copy__(self) -> BedrockLevelDat: ...
    def __deepcopy__(self, memo: dict) -> BedrockLevelDat: ...
    @typing.overload
    def __init__(self) -> None: ...
    @typing.overload
    def __init__(
        self, version: typing.SupportsInt, named_tag: amulet.nbt.NamedTag
    ) -> None: ...
    def __repr__(self) -> str: ...
    def save_to(self, path: os.PathLike | str | bytes) -> None: ...
    def to_binary(self) -> bytes: ...
    @property
    def version(self) -> int: ...
    @version.setter
    def version(self, version: typing.SupportsInt) -> None: ...



from __future__ import annotations

import collections.abc

import amulet.nbt

__all__: list[str] = ["BedrockRawChunk"]

class BedrockRawChunk:
    @property
    def actors(self) -> collections.abc.MutableSequence[amulet.nbt.NamedTag]: ...
    @property
    def data(self) -> collections.abc.MutableMapping[bytes, bytes]: ...



from __future__ import annotations

from amulet.level.abc.level import Level
from amulet.level.loader import NoValidLevelLoader, get_level

from . import _amulet_level, _version, abc, bedrock, java, loader

__all__: list[str] = [
    "Level",
    "NoValidLevelLoader",
    "abc",
    "bedrock",
    "compiler_config",
    "get_level",
    "java",
    "loader",
]

def _init() -> None: ...

__version__: str
compiler_config: dict



import os
import subprocess
from tempfile import TemporaryDirectory


def main() -> str:
    if subprocess.run(["cmake", "--version"]).returncode:
        raise RuntimeError("Could not find cmake")

    with TemporaryDirectory() as build_dir:
        # get the compiler id and version
        if subprocess.run(
            ["cmake", "-S", os.path.dirname(__file__), "-B", build_dir]
        ).returncode:
            raise RuntimeError(
                "Could not find a C++ 20 compiler. Do you have a C++ 20 compiler installed?"
            )

        # Get the compiler variables generated by the cmake file
        with open(os.path.join(build_dir, "compiler_id.txt")) as f:
            compiler_id_str = f.read().strip()
        with open(os.path.join(build_dir, "compiler_version.txt")) as f:
            compiler_version = f.read().strip().split(".", 1)[0]

    # convert the compiler id to an int so it can be used in a version number
    compiler_id_int = 0
    for b in compiler_id_str.encode("utf-8"):
        compiler_id_int <<= 8
        compiler_id_int += b

    # combine the compiler id and compiler version into a version number
    return f"==4.{compiler_id_int}.{compiler_version}"



from __future__ import annotations

import datetime
import os

import amulet.core.version
import amulet.level.abc.registry
import amulet.level.java.raw_dimension
import amulet.nbt
import amulet.utils.event
import amulet.utils.lock
import PIL.Image

__all__: list[str] = ["JavaCreateArgsV1", "JavaRawLevel"]

class JavaCreateArgsV1:
    def __init__(
        self,
        overwrite: bool,
        path: os.PathLike | str | bytes,
        version: amulet.core.version.VersionNumber,
        level_name: str,
    ) -> None: ...
    @property
    def level_name(self) -> str: ...
    @property
    def overwrite(self) -> bool: ...
    @property
    def path(self) -> str: ...
    @property
    def version(self) -> amulet.core.version.VersionNumber: ...

class JavaRawLevel:
    @staticmethod
    def create(args: JavaCreateArgsV1) -> JavaRawLevel:
        """
        Create a new Java level at the given directory.
        Thread safe.
        """

    @staticmethod
    def load(path: os.PathLike | str | bytes) -> JavaRawLevel:
        """
        Load an existing Java level from the given directory.
        Thread safe.
        """

    def close(self) -> None:
        """
        Close the level.
        closed event will be emitted when complete.
        External ReadWrite:Unique lock required.
        """

    def compact(self) -> None:
        """
        Compact the level.
        External Read:SharedReadWrite lock required.
        """

    def get_dimension(
        self, dimension_id: str
    ) -> amulet.level.java.raw_dimension.JavaRawDimension:
        """
        Get the raw dimension object for a specific dimension.
        External Read:SharedReadWrite lock required.
        """

    def is_open(self) -> bool:
        """
        Is the level open.
        External Read:SharedReadWrite lock required.
        """

    def is_supported(self) -> bool:
        """
        Is this level a supported version.
        This is true for all versions we support and false for snapshots and unsupported newer versions.
        TODO: thread safety
        """

    def open(self) -> None:
        """
        Open the level.
        opened event will be emitted when complete.
        External ReadWrite:Unique lock required.
        """

    def reload(self) -> None:
        """
        Reload the level.
        This is like closing and re-opening without releasing the session.lock file.
        External ReadWrite:Unique lock required.
        """

    def reload_metadata(self) -> None:
        """
        Reload the metadata. This can only be called when the level is closed.
        External ReadWrite:Unique lock required.
        """

    @property
    def biome_id_override(self) -> amulet.level.abc.registry.IdRegistry:
        """
        Overridden biome ids.
        External Read:SharedReadWrite lock required.
        """

    @property
    def block_id_override(self) -> amulet.level.abc.registry.IdRegistry:
        """
        Overridden block ids.
        External Read:SharedReadWrite lock required.
        """

    @property
    def closed(self) -> amulet.utils.event.Event[()]: ...
    @property
    def data_version(self) -> amulet.core.version.VersionNumber:
        """
        Getter:
        The game data version that the level was last opened in.
        External Read:SharedReadWrite lock required.

        Setter:
        Set the maximum game version.
        If the game version is different this will call :meth:`reload`.
        External ReadWrite:SharedReadWrite lock required.
        """

    @data_version.setter
    def data_version(self, arg1: amulet.core.version.VersionNumber) -> None: ...
    @property
    def dimension_ids(self) -> list[str]:
        """
        The identifiers for all dimensions in this level.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    @property
    def level_dat(self) -> amulet.nbt.NamedTag:
        """
        Getter:
        The NamedTag stored in the level.dat file. Returns a unique copy.
        External Read:SharedReadWrite lock required.

        Setter:
        Set the level.dat NamedTag
        This calls :meth:`reload` if the data version changed.
        External ReadWrite:Unique lock required.
        """

    @level_dat.setter
    def level_dat(self, arg1: amulet.nbt.NamedTag) -> None: ...
    @property
    def level_name(self) -> str:
        """
        Getter:
        The name of the level.
        External Read:SharedReadWrite lock required.

        Setter:
        Set the level name.
        External ReadWrite:Unique lock required.
        """

    @level_name.setter
    def level_name(self, arg1: str) -> None: ...
    @property
    def lock(self) -> amulet.utils.lock.OrderedLock:
        """
        The public lock
        Thread safe.
        """

    @property
    def modified_time(self) -> datetime.datetime:
        """
        The time when the level was lasted edited.
        External Read:SharedReadWrite lock required.
        """

    @property
    def opened(self) -> amulet.utils.event.Event[()]: ...
    @property
    def path(self) -> str:
        """
        The path to the level directory.
        Thread safe.
        """

    @property
    def platform(self) -> str:
        """
        The platform identifier. "java"
        Thread safe.
        """

    @property
    def reloaded(self) -> amulet.utils.event.Event[()]: ...
    @property
    def thumbnail(self) -> PIL.Image.Image:
        """
        Get the thumbnail for the level.
        Thread safe.
        """



from __future__ import annotations

import typing
from builtins import str as JavaInternalDimensionID

import amulet.level.abc.dimension
import amulet.level.java.chunk_handle

__all__: list[str] = ["JavaDimension", "JavaInternalDimensionID"]

class JavaDimension(amulet.level.abc.dimension.Dimension):
    def get_chunk_handle(
        self, cx: typing.SupportsInt, cz: typing.SupportsInt
    ) -> amulet.level.java.chunk_handle.JavaChunkHandle:
        """
        Get the chunk handle for the given chunk in this dimension.
        Thread safe.

        :param cx: The chunk x coordinate to load.
        :param cz: The chunk z coordinate to load.
        """



from __future__ import annotations

import datetime

import amulet.core.version
import amulet.level.abc.dimension
import amulet.utils.event
import amulet.utils.lock
import PIL.Image

__all__: list[str] = [
    "CompactibleLevel",
    "DiskLevel",
    "Level",
    "LevelMetadata",
    "ReloadableLevel",
]

class CompactibleLevel:
    def compact(self) -> None:
        """
        Compact the level data to reduce file size.
        External ReadWrite:SharedReadWrite lock required.
        """

class DiskLevel:
    @property
    def path(self) -> str:
        """
        The path to the level on disk.
        External Read:SharedReadWrite lock required.
        """

class Level(LevelMetadata):
    def close(self) -> None:
        """
        Close the level.
        External ReadWrite:Unique lock required.

        If the level is not open, this does nothing.
        """

    def create_restore_point(self) -> None:
        """
        Create a new history restore point.
        Any changes made after this point can be reverted by calling undo.
        External Read:SharedReadWrite lock required.
        """

    def dimension_ids(self) -> list[str]:
        """
        The identifiers for all dimensions in the level.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    def get_dimension(self, dimension_id: str) -> amulet.level.abc.dimension.Dimension:
        """
        Get a dimension.
        External Read:SharedReadWrite lock required.
        External ReadWrite:SharedReadWrite lock required when calling code in Dimension (and its children) that need write permission.
        """

    def get_redo_count(self) -> int:
        """
        Get the number of times redo can be called.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    def get_undo_count(self) -> int:
        """
        Get the number of times undo can be called.
        External Read:SharedReadWrite lock required.
        External Read:SharedReadOnly lock optional.
        """

    def open(self) -> None:
        """
        Open the level.

        If the level is already open, this does nothing.
        External ReadWrite:Unique lock required.
        """

    def purge(self) -> None:
        """
        Clear all unsaved changes and restore points.
        External ReadWrite:Unique lock required.
        """

    def redo(self) -> None:
        """
        Redo changes that were previously reverted.
        External ReadWrite:SharedReadWrite lock required.
        External ReadWrite:Unique lock optional.
        """

    def save(self) -> None:
        """
        Save all changes to the level.
        External ReadWrite:Unique lock required.
        """

    def undo(self) -> None:
        """
        Revert the changes made since the previous restore point.
        External ReadWrite:SharedReadWrite lock required.
        External ReadWrite:Unique lock optional.
        """

    @property
    def closed(self) -> amulet.utils.event.Event[()]:
        """
        Event emitted when the level is closed.
        Thread safe.
        """

    @property
    def history_changed(self) -> amulet.utils.event.Event[()]:
        """
        An event emitted when the undo or redo count changes.
        Thread safe.
        """

    @property
    def history_enabled(self) -> bool:
        """
        A boolean tracking if the history system is enabled.
        External Read:SharedReadWrite lock required when getting.
        External ReadWrite:SharedReadWrite lock required when setting.

        If true, the caller must call :meth:`create_restore_point` before making changes.
        :attr:`history_enabled_changed` is emitted when this is set.
        """

    @history_enabled.setter
    def history_enabled(self, arg1: bool) -> None: ...
    @property
    def history_enabled_changed(self) -> amulet.utils.event.Event[()]:
        """
        An event emitted when set_history_enabled is called.
        Thread safe.
        """

    @property
    def opened(self) -> amulet.utils.event.Event[()]:
        """
        Event emitted when the level is opened.
        Thread safe.
        """

    @property
    def purged(self) -> amulet.utils.event.Event[()]:
        """
        Event emitted when the level is purged
        Thread safe.
        """

class LevelMetadata:
    def is_open(self) -> bool:
        """
        Has the level been opened.
        External Read:SharedReadWrite lock required.

        :return: True if the level is open otherwise False.
        """

    def is_supported(self) -> bool:
        """
        Is this level a supported version.
        This is true for all versions we support and false for snapshots, betas and unsupported newer versions.
        External Read:SharedReadWrite lock required.
        """

    @property
    def level_name(self) -> str:
        """
        The name of the level
        External Read:SharedReadWrite lock required.
        """

    @property
    def lock(self) -> amulet.utils.lock.OrderedLock:
        """
        The external mutex for the level.
        Thread safe.
        """

    @property
    def max_block_version(self) -> amulet.core.version.VersionNumber:
        """
        Get the suggested maximum block version this level can accept.
        Note that on some platforms the real max version may be higher.
        External Read:SharedReadWrite lock required.
        """

    @property
    def max_game_version(self) -> amulet.core.version.VersionNumber:
        """
        The maximum game version the level has been opened with.
        External Read:SharedReadWrite lock required.
        """

    @property
    def modified_time(self) -> datetime.datetime:
        """
        The time when the level was last modified.
        External Read:SharedReadWrite lock required.
        """

    @property
    def platform(self) -> str:
        """
        The platform string for the level.
        External Read:SharedReadWrite lock required.
        """

    @property
    def sub_chunk_size(self) -> int:
        """
        The size of the sub-chunk. Must be a cube.
        External Read:SharedReadWrite lock required.
        """

    @property
    def thumbnail(self) -> PIL.Image.Image:
        """
        The thumbnail for the level.
        External Read:SharedReadWrite lock required.
        """

class ReloadableLevel:
    def reload(self) -> None:
        """
        Reload the level.
        This is like closing and opening the level but does not release locks.
        This can only be done when the level is open.External ReadWrite:Unique lock required.
        """

    def reload_metadata(self) -> None:
        """
        Reload the level metadata.
        This can only be done when the level is not open.
        External ReadWrite:Unique lock required.
        """

    @property
    def reloaded(self) -> amulet.utils.event.Event[()]:
        """
        Event emitted when the level is reloaded.
        Thread safe.
        """
